-- ========================================================================== --
--                    Boss Mod Nameplates                                     --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio              "BossModNameplates.BigWigs"                         "1.1.0"
-- ========================================================================== --
namespace "BMN"
-- ========================================================================== --
__AttributeUsage__{AttributeTarget = AttributeTargets.ObjectMethod, RunOnce = true, AllowMultiple = true}
__Sealed__()
class "__BigWigsEvent__" (function(_ENV)
    extend "IAttribute"

    _BigWigsHandlers = {}

    function _BigWigsHandlers:ConvertToScorpioMsg(e, ...)
        Scorpio.FireSystemEvent(e, ...)
    end

    local function RegisterBigWigsEvent(name)
        if BigWigsLoader and not _BigWigsHandlers[name] then
            _BigWigsHandlers[name] = true
            BigWigsLoader.RegisterMessage(_BigWigsHandlers, name, "ConvertToScorpioMsg")
        end
    end

    -- target     - the method
    -- targetType - AttributeTargets.ObjectMethod
    -- owner      - The object
    -- name       - The method's name
    function ApplyAttribute(self, target, targetType, owner, name)
        if getmetatable(owner) == Scorpio then
            -- Here is the details
            if #self > 0 then
                -- If the event list applied
                for _, evt in ipairs(self) do
                    RegisterBigWigsEvent(evt)
                    owner:RegisterEvent(evt, target)
                end
            else
                -- If no event list existed, use the method name
                RegisterBigWigsEvent(name)
                owner:RegisterEvent(name, target)
            end
        else
            error("__BigWigsEvent__ can only be applyed to objects of Scorpio.")
        end
    end

    ----------------------------------------------
    ----------------- Constructor ----------------
    ----------------------------------------------
    -- So it support __BigWigsEvent__ "XXXX",  or __BigWigsEvent__("XXXX", "XXX")
    __Arguments__{ Argument(NEString, true, nil, nil, true) }
    function __BigWigsEvent__(self, ...)
        -- If the attribute is used to register something with the method
        -- the Priorty must be lowest
        -- BTW, the __Thread__ is lower priorty, since it will wrap the method
        self.Priorty = AttributePriorty.Lowest

        for i = 1, select('#', ...) do
            tinsert(self, select(i, ...))
        end
    end

    ----------------------------------------------
    ----------------- Meta-Method ----------------
    ----------------------------------------------
    -- So it support __BigWigsEvent__ "XXXX" "XXXX"
    __Arguments__{ NEString }
    function __call(self, other)
        tinsert(self, other)
        return self
    end

    -- Make sure multi attr can be appled
    function __eq(self, other) return false end
end)

-- ========================================================================== --
function OnLoad(self)
  if BigWigsLoader then
    Debug("[BigWigs] is Loaded")
  end
end

function OnEnable(self)

end

function OnDisable(self)

end

-- ========================================================================== --
-- BigWigs Support                                                            --
-- ========================================================================== --
__BigWigsEvent__()
function BigWigs_ShowNameplateAura(_, playerName, icon, duration, desaturate)
  Trace("[BigWigs] Show nameplate aura")
  local aura = Aura()
  aura.guid = UnitGUID(playerName)
  aura.unit = playerName
  aura.duration = duration or 0
  aura.desaturated = desaturate
  aura.icon = icon
  aura.startTime = GetTime()
  aura.endTime = aura.startTime + aura.duration
  aura.borderColor = { r = 0, g = 0, b = 0 }

  if _Addon.isInEncounter then
    Encounter.ApplyCustomChange(aura, _Addon.encounterID, _Addon.encounterDifficulty, BossMod.BigWigs)
  end

  _Addon:ShowNameplateAura(aura)
end

__BigWigsEvent__()
function BigWigs_HideNameplateAura(_, playerName)
  Trace("[BigWigs] Hide namplate Aura")
  _Addon:HideNameplateAura(UnitGUID(playerName))
end
