-- ========================================================================== --
--                    Boss Mod Nameplates                               --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio                "BossModNameplates.DBM"                           "1.1.0"
-- ========================================================================== --
namespace "BMN"
-- ========================================================================== --
__AttributeUsage__{AttributeTarget = AttributeTargets.ObjectMethod, RunOnce = true, AllowMultiple = true}
__Sealed__()
class "__DBMEvent__" (function(_ENV)
    extend "IAttribute"

    _DBMHandlers = {}

    function _DBMHandlers:ConvertToScorpioMsg(e, ...)
        Scorpio.FireSystemEvent(e, ...)
    end

    local function RegisterDBMEvent(name)
        if DBM and not _DBMHandlers[name] then
          _DBMHandlers[name] = true
          DBM:RegisterCallback(name, function(msg, ...) _DBMHandlers:ConvertToScorpioMsg(msg, ...) end)
        end
    end

    -- target     - the method
    -- targetType - AttributeTargets.ObjectMethod
    -- owner      - The object
    -- name       - The method's name
    function ApplyAttribute(self, target, targetType, owner, name)
        if getmetatable(owner) == Scorpio then
            -- Here is the details
            if #self > 0 then
                -- If the event list applied
                for _, evt in ipairs(self) do
                    RegisterDBMEvent(evt)
                    owner:RegisterEvent(evt, target)
                end
            else
                -- If no event list existed, use the method name
                RegisterDBMEvent(name)
                owner:RegisterEvent(name, target)
            end
        else
            error("__DBMEvent__ can only be applyed to objects of Scorpio.")
        end
    end

    ----------------------------------------------
    ----------------- Constructor ----------------
    ----------------------------------------------
    -- So it support __DBMEvent__ "XXXX",  or __DBMEvent__("XXXX", "XXX")
    __Arguments__{ Argument(NEString, true, nil, nil, true) }
    function __DBMEvent__(self, ...)
        -- If the attribute is used to register something with the method
        -- the Priorty must be lowest
        -- BTW, the __Thread__ is lower priorty, since it will wrap the method
        self.Priorty = AttributePriorty.Lowest

        for i = 1, select('#', ...) do
            tinsert(self, select(i, ...))
        end
    end

    ----------------------------------------------
    ----------------- Meta-Method ----------------
    ----------------------------------------------
    -- So it support __DBMEvent__ "XXXX" "XXXX"
    __Arguments__{ NEString }
    function __call(self, other)
        tinsert(self, other)
        return self
    end

    -- Make sure multi attr can be appled
    function __eq(self, other) return false end
end)

-- ========================================================================== --
function OnLoad(self)
  if DBM then
    Debug("[DBM] is loaded")
    DBM.Nameplate.SupportedNPMod = function() return true end -- say to DBM we support it
  end
end

function OnEnable(self)

end

function OnDisable(self)

end
-- ========================================================================== --
-- DBM Support
-- ========================================================================== --
__DBMEvent__()
function BossMod_ShowNameplateAura(isGUID, unit, currentTexture, duration, desaturate)
  Trace("[DBM] Show nameplate aura")
  print("BossMod_ShowNameplateAura")
  print("isGUID", isGUID)
  print("unit", unit)
  print("currentTexture", currentTexture)
  print("duration", duration)
  print("desaturate", desaturate)

  local aura = Aura()
  aura.guid = isGUID and unit or UnitGUID(unit)
  aura.unit = not isGUID and unit
  aura.icon = currentTexture
  aura.duration = duration or 0
  aura.desaturated = desaturate
  aura.startTime = GetTime()
  aura.endTime = aura.startTime + aura.duration
  aura.borderColor = { r = 0, g = 0, b = 0 }

  if _Addon.isInEncounter then
    Encounter.ApplyCustomChange(aura, _Addon.encounterID, _Addon.encounterDifficulty, BossMod.DBM)
  end

  _Addon:ShowNameplateAura(aura)
end

__DBMEvent__()
function BossMod_HideNameplateAura(isGUID, unit, currentTexture)
  Trace("[DBM] Hide nameplate aura")
  print("BossMod_HideNameplateAura")
  print("isGUID", isGUID)
  print("unit", unit)
  print("currentTexture", currentTexture)
  local guid = isGUID and unit or UnitGUID(unit)
  _Addon:HideNameplateAura(guid)

end
